@@ -1,4 +1,4 @@
-/* $Id: files.c 5481 2015-12-05 11:38:26Z bens $ */
+/* $Id: files.c 5519 2015-12-31 19:20:40Z bens $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
@@ -323,7 +323,7 @@ int do_lockfile(const char *filename)
 
 /* If it's not "", filename is a file to open.  We make a new buffer, if
  * necessary, and then open and read the file, if applicable. */
-void open_buffer(const char *filename, bool undoable)
+bool open_buffer(const char *filename, bool undoable)
 {
     bool quiet = FALSE;
     bool new_buffer = (openfile == NULL
@@ -343,7 +343,7 @@ void open_buffer(const char *filename, bool undoable)
     if (check_operating_dir(filename, FALSE)) {
 	statusbar(_("Can't insert file from outside of %s"),
 		operating_dir);
-	return;
+	return FALSE;
     }
 #endif
 
@@ -358,7 +358,7 @@ void open_buffer(const char *filename, bool undoable)
 	    else
 		statusbar(_("\"%s\" is not a normal file"), filename);
 	    beep();
-	    return;
+	    return FALSE;
 	}
     }
 
@@ -374,7 +374,7 @@ void open_buffer(const char *filename, bool undoable)
 #ifndef DISABLE_MULTIBUFFER
 		if (openfile->next) {
 		    close_buffer(TRUE);
-		    return;
+		    return FALSE;
 		}
 #endif
 	    } else if (lockstatus == 0) {
@@ -421,6 +421,7 @@ void open_buffer(const char *filename, bool undoable)
     if (new_buffer)
 	color_update();
 #endif
+    return TRUE;
 }
 
 #ifndef DISABLE_SPELLER
@@ -456,9 +457,14 @@ void display_buffer(void)
     titlebar(NULL);
 
 #ifndef DISABLE_COLOR
-    /* Make sure we're using the buffer's associated colors, if
-     * applicable. */
+    /* Make sure we're using the buffer's associated colors. */
     color_init();
+
+    /* If there are multiline coloring regexes, and there is no
+     * multiline cache data yet, precalculate it now. */
+    if (openfile->syntax && openfile->syntax->nmultis > 0 &&
+		openfile->fileage->multidata == NULL)
+	precalc_multicolorinfo();
 #endif
 
     /* Update the edit window. */
@@ -466,23 +472,21 @@ void display_buffer(void)
 }
 
 #ifndef DISABLE_MULTIBUFFER
-/* Switch to the next file buffer if next_buf is TRUE.  Otherwise,
- * switch to the previous file buffer. */
-void switch_to_prevnext_buffer(bool next_buf, bool quiet)
+/* Switch to a neighbouring file buffer; to the next if to_next is TRUE;
+ * otherwise, to the previous one. */
+void switch_to_prevnext_buffer(bool to_next, bool quiet)
 {
     assert(openfile != NULL);
 
-    /* If only one file buffer is open, indicate it on the statusbar and
-     * get out. */
+    /* If only one file buffer is open, say so and get out. */
     if (openfile == openfile->next) {
-	if (quiet == FALSE)
+	if (!quiet)
 	    statusbar(_("No more open file buffers"));
 	return;
     }
 
-    /* Switch to the next or previous file buffer, depending on the
-     * value of next_buf. */
-    openfile = next_buf ? openfile->next : openfile->prev;
+    /* Switch to the next or previous file buffer. */
+    openfile = to_next ? openfile->next : openfile->prev;
 
 #ifdef DEBUG
     fprintf(stderr, "filename is %s\n", openfile->filename);
@@ -492,7 +496,7 @@ void switch_to_prevnext_buffer(bool next_buf, bool quiet)
     display_buffer();
 
     /* Indicate the switch on the statusbar. */
-    if (quiet == FALSE)
+    if (!quiet)
 	statusbar(_("Switched to %s"),
 		((openfile->filename[0] == '\0') ?
 		_("New Buffer") : openfile->filename));
@@ -585,12 +589,9 @@ int is_file_writable(const char *filename)
     return ans;
 }
 
-/* We make a new line of text from buf.  buf is length buf_len.  If
- * first_line_ins is TRUE, then we put the new line at the top of the
- * file.  Otherwise, we assume prevnode is the last line of the file,
- * and put our line after prevnode. */
-filestruct *read_line(char *buf, filestruct *prevnode, bool
-	*first_line_ins, size_t buf_len)
+/* Make a new line of text from the given buf, which is of length buf_len.
+ * Then attach this line after prevnode. */
+filestruct *read_line(char *buf, size_t buf_len, filestruct *prevnode)
 {
     filestruct *fileptr = (filestruct *)nmalloc(sizeof(filestruct));
 
@@ -612,20 +613,17 @@ filestruct *read_line(char *buf, filestruct *prevnode, bool
     fileptr->multidata = NULL;
 #endif
 
-    if (*first_line_ins) {
+    fileptr->prev = prevnode;
+
+    if (prevnode == NULL) {
 	/* Special case: we're inserting into the first line. */
-	fileptr->prev = NULL;
 	fileptr->next = openfile->fileage;
 	openfile->fileage = fileptr;
 	fileptr->lineno = 1;
 	/* Make sure that our edit window stays on the first line. */
 	openfile->edittop = fileptr;
-	*first_line_ins = FALSE;
     } else {
-	assert(prevnode != NULL);
-
 	prevnode->next = fileptr;
-	fileptr->prev = prevnode;
 	fileptr->next = NULL;
 	fileptr->lineno = prevnode->lineno + 1;
     }
@@ -652,10 +650,8 @@ void read_file(FILE *f, int fd, const char *filename, bool undoable, bool checkw
 	/* The current input character. */
     char *buf;
 	/* The buffer where we store chunks of the file. */
-    filestruct *fileptr = openfile->current;
-	/* The current line of the file. */
-    bool first_line_ins = FALSE;
-	/* Whether we're inserting with the cursor on the first line. */
+    filestruct *fileptr = openfile->current->prev;
+	/* The line after which to start inserting. */
     int input_int;
 	/* The current value we read from the file, whether an input
 	 * character or EOF. */
@@ -676,11 +672,6 @@ void read_file(FILE *f, int fd, const char *filename, bool undoable, bool checkw
 	add_undo(INSERT);
 #endif
 
-    if (openfile->current == openfile->fileage)
-	first_line_ins = TRUE;
-    else
-	fileptr = openfile->current->prev;
-
     /* Read the entire file into the filestruct. */
     while ((input_int = getc(f)) != EOF) {
 	input = (char)input_int;
@@ -701,7 +692,7 @@ void read_file(FILE *f, int fd, const char *filename, bool undoable, bool checkw
 #endif
 
 	    /* Read in the line properly. */
-	    fileptr = read_line(buf, fileptr, &first_line_ins, len);
+	    fileptr = read_line(buf, len, fileptr);
 
 	    /* Reset the line length in preparation for the next line. */
 	    len = 0;
@@ -722,7 +713,7 @@ void read_file(FILE *f, int fd, const char *filename, bool undoable, bool checkw
 		format += 2;
 
 	    /* Read in the line properly. */
-	    fileptr = read_line(buf, fileptr, &first_line_ins, len);
+	    fileptr = read_line(buf, len, fileptr);
 
 	    /* Reset the line length in preparation for the next line.
 	     * Since we've already read in the next character, reset it
@@ -788,17 +779,12 @@ void read_file(FILE *f, int fd, const char *filename, bool undoable, bool checkw
 #endif
 
 	/* Read in the last line properly. */
-	fileptr = read_line(buf, fileptr, &first_line_ins, len);
+	fileptr = read_line(buf, len, fileptr);
 	num_lines++;
     }
 
     free(buf);
 
-    /* If we didn't get a file and we don't already have one, open a
-     * blank buffer. */
-    if (fileptr == NULL)
-	open_buffer("", FALSE);
-
     /* Attach the file we got to the filestruct.  If we got a file of
      * zero bytes, don't do anything. */
     if (num_lines > 0) {
@@ -1221,7 +1207,7 @@ void do_insertfile(
 			!execute &&
 #endif
 			check_poshistory(answer, &savedposline, &savedposcol))
-		    do_gotolinecolumn(savedposline, savedposcol, FALSE, FALSE, FALSE, FALSE);
+		    do_gotolinecolumn(savedposline, savedposcol, FALSE, FALSE);
 	    } else
 #endif /* !DISABLE_MULTIBUFFER && !DISABLE_HISTORIES */
 	    {
@@ -2221,11 +2207,11 @@ bool write_marked_file(const char *name, FILE *f_open, bool tmp,
 #endif /* !NANO_TINY */
 
 /* Write the current file to disk.  If the mark is on, write the current
- * marked selection to disk.  If exiting is TRUE, write the file to disk
- * regardless of whether the mark is on, and without prompting if the
- * TEMP_FILE flag is set and the current file has a name.  Return TRUE
- * on success or FALSE on error. */
-bool do_writeout(bool exiting)
+ * marked selection to disk.  If exiting is TRUE, write the entire file
+ * to disk regardless of whether the mark is on, and without prompting if
+ * the TEMP_FILE flag is set and the current file has a name.  Return 0
+ * on error, 1 on success, and 2 when the buffer is to be discarded. */
+int do_writeout(bool exiting)
 {
     int i;
     append_type append = OVERWRITE;
@@ -2234,15 +2220,13 @@ bool do_writeout(bool exiting)
 #ifndef DISABLE_EXTRA
     static bool did_credits = FALSE;
 #endif
-    bool retval = FALSE;
+    bool result = FALSE;
 
-    if (openfile->filename[0] != '\0' && ISSET(TEMP_FILE)) {
-	retval = write_file(openfile->filename, NULL, FALSE, OVERWRITE,
-		FALSE);
+    if (exiting && openfile->filename[0] != '\0' && ISSET(TEMP_FILE)) {
+	result = write_file(openfile->filename, NULL, FALSE, OVERWRITE, FALSE);
 
-	/* Write succeeded. */
-	if (retval)
-	    return retval;
+	if (result)
+	    return 1;	/* The write succeeded. */
     }
 
     ans = mallocstrcpy(NULL,
@@ -2275,14 +2259,6 @@ bool do_writeout(bool exiting)
 		  (append == APPEND) ? _("File Name to Append to") :
 		  _("File Name to Write");
 
-	/* If we're not exiting, and the TEMP_FILE flag is set, and
-	 * the current file doesn't have a name, warn the user before
-	 * prompting for a name.  If we are exiting, we've already
-	 * warned the user just before the "Save modified buffer"
-	 * prompt, so we don't need to do it again. */
-	if (!exiting && openfile->filename[0] == '\0' && ISSET(TEMP_FILE))
-	    no_current_file_name_warning();
-
 	/* If we're using restricted mode, the filename isn't blank,
 	 * and we're at the "Write File" prompt, disable tab
 	 * completion. */
@@ -2307,11 +2283,25 @@ bool do_writeout(bool exiting)
 	 * encoded null), treat it as though it's blank. */
 	if (i < 0 || *answer == '\n') {
 	    statusbar(_("Cancelled"));
-	    retval = FALSE;
 	    break;
 	} else {
 	    functionptrtype func = func_from_key(&i);
 
+	    /* Upon request, abandon the buffer, if user is sure. */
+	    if (func == discard_buffer) {
+		if (openfile->modified)
+		    i = do_yesno_prompt(FALSE,
+				_("Save modified buffer anyway ? "));
+		else
+		    i = 0;
+
+		if (i == 0) {
+		    free(ans);
+		    return 2;	/* Yes, discard the buffer. */
+		} else
+		    continue;	/* Go back to the filename prompt. */
+	    }
+
 	    ans = mallocstrcpy(ans, answer);
 
 #ifndef DISABLE_BROWSER
@@ -2365,7 +2355,6 @@ bool do_writeout(bool exiting)
 			strcasecmp(answer, "zzy") == 0) {
 		do_credits();
 		did_credits = TRUE;
-		retval = FALSE;
 		break;
 	    }
 #endif
@@ -2449,7 +2438,7 @@ bool do_writeout(bool exiting)
 	     * a separate file.  If we're using restricted mode, this
 	     * function is disabled, since it allows reading from or
 	     * writing to files not specified on the command line. */
-	    retval =
+	    result =
 #ifndef NANO_TINY
 		(!ISSET(RESTRICTED) && !exiting && openfile->mark_set) ?
 		write_marked_file(answer, NULL, FALSE, append) :
@@ -2462,14 +2451,16 @@ bool do_writeout(bool exiting)
 
     free(ans);
 
-    return retval;
+    return result ? 1 : 0;
 }
 
-/* Write the current file to disk.  If the mark is on, write the current
- * marked selection to disk. */
+/* Write the current buffer to disk, or discard it. */
 void do_writeout_void(void)
 {
-    do_writeout(FALSE);
+    /* If the user chose to discard the buffer, close it. */
+    if (do_writeout(FALSE) == 2)
+	close_and_go();
+
     display_main_list();
 }
 
@@ -3095,15 +3086,15 @@ void save_history(void)
 	FILE *hist = fopen(nanohist, "wb");
 
 	if (hist == NULL)
-	    history_error(N_("Error writing %s: %s"), nanohist,
-		strerror(errno));
+	    fprintf(stderr, _("Error writing %s: %s\n"), nanohist,
+			strerror(errno));
 	else {
 	    /* Make sure no one else can read from or write to the
 	     * history file. */
 	    chmod(nanohist, S_IRUSR | S_IWUSR);
 
 	    if (!writehist(hist, searchage) || !writehist(hist, replaceage))
-		history_error(N_("Error writing %s: %s"), nanohist,
+		fprintf(stderr, _("Error writing %s: %s\n"), nanohist,
 			strerror(errno));
 
 	    fclose(hist);
@@ -3126,8 +3117,8 @@ void save_poshistory(void)
 	FILE *hist = fopen(poshist, "wb");
 
 	if (hist == NULL)
-	    history_error(N_("Error writing %s: %s"), poshist,
-		strerror(errno));
+	    fprintf(stderr, _("Error writing %s: %s\n"), poshist,
+			strerror(errno));
 	else {
 	    /* Make sure no one else can read from or write to the
 	     * history file. */
@@ -3138,8 +3129,8 @@ void save_poshistory(void)
 		sprintf(statusstr, "%s %ld %ld\n", posptr->filename, (long)posptr->lineno,
 			(long)posptr->xno);
 		if (fwrite(statusstr, sizeof(char), strlen(statusstr), hist) < strlen(statusstr))
-		    history_error(N_("Error writing %s: %s"), poshist,
-			strerror(errno));
+		    fprintf(stderr, _("Error writing %s: %s\n"), poshist,
+				strerror(errno));
 		free(statusstr);
 	    }
 	    fclose(hist);
