@@ -1,4 +1,4 @@
-/* $Id: global.c 5434 2015-11-23 08:52:23Z bens $ */
+/* $Id: global.c 5537 2016-01-07 15:42:44Z bens $ */
 /**************************************************************************
  *   global.c                                                             *
  *                                                                        *
@@ -270,6 +270,9 @@ void prepend_void(void)
 void backup_file_void(void)
 {
 }
+void discard_buffer(void)
+{
+}
 void new_buffer_void(void)
 {
 }
@@ -438,20 +441,21 @@ void assign_keyinfo(sc *s)
 	    s->seq = KEY_DC;
 	else if (!strcasecmp(s->keystr, "Bsp"))
 	    s->seq = KEY_BACKSPACE;
+	/* The Tab and Enter keys don't actually produce special codes
+	 * but the exact integer values of ^I and ^M.  Rebinding the
+	 * latter therefore also rebinds Tab and Enter. */
+	else if (!strcasecmp(s->keystr, "Tab"))
+	    s->seq = NANO_CONTROL_I;
 	else if (!strcasecmp(s->keystr, "Enter"))
 	    s->seq = KEY_ENTER;
 	else if (!strcasecmp(s->keystr, "PgUp"))
 	    s->seq = KEY_PPAGE;
 	else if (!strcasecmp(s->keystr, "PgDn"))
 	    s->seq = KEY_NPAGE;
-#ifdef KEY_HOME
 	else if (!strcasecmp(s->keystr, "Home"))
 	    s->seq = KEY_HOME;
-#endif
-#ifdef KEY_END
 	else if (!strcasecmp(s->keystr, "End"))
 	    s->seq = KEY_END;
-#endif
     }
 }
 
@@ -632,6 +636,7 @@ void shortcut_init(void)
     const char *nano_backup_msg = N_("Toggle backing up of the original file");
     const char *nano_execute_msg = N_("Execute external command");
 #endif
+    const char *nano_discard_buffer_msg = N_("Close buffer without saving it");
 #ifndef DISABLE_MULTIBUFFER
     const char *nano_multibuffer_msg = N_("Toggle the use of a new buffer");
 #endif
@@ -659,12 +664,6 @@ void shortcut_init(void)
 #define IFSCHELP(help) ""
 #endif
 
-    while (allfuncs != NULL) {
-	subnfunc *f = allfuncs;
-	allfuncs = allfuncs->next;
-	free(f);
-    }
-
     /* Start populating the different menus with functions. */
 
     add_to_funcs(do_help_void, MMOST,
@@ -1006,6 +1005,9 @@ void shortcut_init(void)
 	N_("Last File"), IFSCHELP(nano_lastfile_msg), BLANKAFTER, VIEW);
 #endif
 
+    add_to_funcs(discard_buffer, MWRITEFILE,
+	N_("Discard buffer"), IFSCHELP(nano_discard_buffer_msg), BLANKAFTER, NOVIEW);
+
 #if !defined(NANO_TINY) && !defined(DISABLE_BROWSER)
     add_to_funcs(do_research, MBROWSER,
 	whereis_next_tag, IFSCHELP(nano_whereis_next_msg), TOGETHER, VIEW);
@@ -1197,6 +1199,8 @@ void shortcut_init(void)
     add_to_sclist(MBROWSER, "M-G", goto_dir_void, 0);
     add_to_sclist(MBROWSER, "F13", goto_dir_void, 0);
 #endif
+    if (ISSET(TEMP_FILE))
+	add_to_sclist(MWRITEFILE, "^Q", discard_buffer, 0);
     add_to_sclist(MWRITEFILE, "M-D", dos_format_void, 0);
     add_to_sclist(MWRITEFILE, "M-M", mac_format_void, 0);
     if (!ISSET(RESTRICTED)) {
@@ -1222,6 +1226,7 @@ void shortcut_init(void)
     add_to_sclist(MHELP, "End", do_last_line, 0);
 #endif
     add_to_sclist(MMOST, "^I", do_tab, 0);
+    add_to_sclist(MMOST, "Tab", do_tab, 0);
     add_to_sclist(MMOST, "^M", do_enter, 0);
     add_to_sclist(MMOST, "Enter", do_enter, 0);
     add_to_sclist(MMOST, "^D", do_delete, 0);
@@ -1257,12 +1262,12 @@ void set_spell_shortcuts(void)
 }
 #endif
 
-const subnfunc *sctofunc(sc *s)
+const subnfunc *sctofunc(const sc *s)
 {
-    subnfunc *f;
+    subnfunc *f = allfuncs;
 
-    for (f = allfuncs; f != NULL && s->scfunc != f->scfunc; f = f->next)
-	;
+    while (f != NULL && f->scfunc != s->scfunc)
+	f = f->next;
 
     return f;
 }
@@ -1318,7 +1323,7 @@ const char *flagtostr(int flag)
 #ifndef DISABLE_NANORC
 /* Interpret a function string given in the rc file, and return a
  * shortcut struct with the corresponding function filled in. */
-sc *strtosc(char *input)
+sc *strtosc(const char *input)
 {
     sc *s;
 
@@ -1333,6 +1338,8 @@ sc *strtosc(char *input)
 	s->scfunc = do_cancel;
     else if (!strcasecmp(input, "exit"))
 	s->scfunc = do_exit;
+    else if (!strcasecmp(input, "discardbuffer"))
+	s->scfunc = discard_buffer;
     else if (!strcasecmp(input, "writeout"))
 	s->scfunc = do_writeout_void;
 #ifndef NANO_TINY
@@ -1569,7 +1576,7 @@ sc *strtosc(char *input)
 }
 
 /* Interpret a menu name and return the corresponding menu flag. */
-int strtomenu(char *input)
+int strtomenu(const char *input)
 {
     if (!strcasecmp(input, "all"))
 	return (MMOST|MHELP|MYESNO);
