@@ -1,4 +1,4 @@
-/* $Id: nano.c 5482 2015-12-06 05:21:43Z astyanax $ */
+/* $Id: nano.c 5542 2016-01-10 16:18:43Z bens $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -31,7 +31,6 @@
 #include <errno.h>
 #include <ctype.h>
 #include <locale.h>
-#include <time.h>
 #ifdef ENABLE_UTF8
 #include <langinfo.h>
 #endif
@@ -104,9 +103,13 @@ void splice_node(filestruct *afterthis, filestruct *newnode)
     if (afterthis->next != NULL)
 	afterthis->next->prev = newnode;
     afterthis->next = newnode;
+
+    /* Update filebot when inserting a node at the end of file. */
+    if (openfile && openfile->filebot == afterthis)
+	openfile->filebot = newnode;
 }
 
-/* Unlink a node from the rest of the filestruct and delete it. */
+/* Disconnect a node from a linked list of filestructs and delete it. */
 void unlink_node(filestruct *fileptr)
 {
     assert(fileptr != NULL);
@@ -116,10 +119,14 @@ void unlink_node(filestruct *fileptr)
     if (fileptr->next != NULL)
 	fileptr->next->prev = fileptr->prev;
 
+    /* Update filebot when removing a node at the end of file. */
+    if (openfile && openfile->filebot == fileptr)
+	openfile->filebot = fileptr->prev;
+
     delete_node(fileptr);
 }
 
-/* Delete a node from the filestruct. */
+/* Free the data structures in the given node. */
 void delete_node(filestruct *fileptr)
 {
     assert(fileptr != NULL && fileptr->data != NULL);
@@ -957,7 +964,7 @@ void usage(void)
 void version(void)
 {
     printf(_(" GNU nano, version %s\n"), VERSION);
-    printf(" (C) 1999..2015 Free Software Foundation, Inc.\n");
+    printf(" (C) 1999..2016 Free Software Foundation, Inc.\n");
     printf(
 	_(" Email: nano@nano-editor.org	Web: http://www.nano-editor.org/"));
     printf(_("\n Compiled options:"));
@@ -1139,25 +1146,29 @@ void do_exit(void)
 
     /* If the user chose not to save, or if the user chose to save and
      * the save succeeded, we're ready to exit. */
-    if (i == 0 || (i == 1 && do_writeout(TRUE))) {
-
-#ifndef NANO_TINY
-	if (ISSET(LOCKING) && openfile->lock_filename)
-	    delete_lockfile(openfile->lock_filename);
-#endif
-
-#ifndef DISABLE_MULTIBUFFER
-	/* Exit only if there are no more open file buffers. */
-	if (!close_buffer(FALSE))
-#endif
-	    finish();
-    /* If the user canceled, we go on. */
-    } else if (i != 1)
+    if (i == 0 || (i == 1 && do_writeout(TRUE)))
+	close_and_go();
+    else if (i != 1)
 	statusbar(_("Cancelled"));
 
     display_main_list();
 }
 
+/* Close the current buffer, and terminate nano if it was the last. */
+void close_and_go(void)
+{
+#ifndef NANO_TINY
+    /* If there is a lockfile, remove it. */
+    if (ISSET(LOCKING) && openfile->lock_filename)
+	delete_lockfile(openfile->lock_filename);
+#endif
+#ifndef DISABLE_MULTIBUFFER
+    /* If there are no more open file buffers, jump off a cliff. */
+    if (!close_buffer(FALSE))
+#endif
+	finish();
+}
+
 /* Another placeholder for function mapping. */
 void do_cancel(void)
 {
@@ -1408,10 +1419,9 @@ void regenerate_screen(void)
     total_refresh();
 }
 
-/* If allow is FALSE, block any SIGWINCH signals that we get, so that
- * we can deal with them later.  If allow is TRUE, unblock any SIGWINCH
- * signals that we have, so that we can deal with them now. */
-void allow_pending_sigwinch(bool allow)
+/* If allow is FALSE, block any SIGWINCH signal.  If allow is TRUE,
+ * unblock SIGWINCH so any pending ones can be dealt with. */
+void allow_sigwinch(bool allow)
 {
     sigset_t winch;
 
@@ -1653,7 +1663,6 @@ int do_input(bool allow_funcs)
 		kbinput = (int *)nrealloc(kbinput, kbinput_len *
 			sizeof(int));
 		kbinput[kbinput_len - 1] = input;
-
 	    }
 	}
 
@@ -1692,6 +1701,7 @@ int do_input(bool allow_funcs)
 	}
 
 	if (have_shortcut) {
+	    const subnfunc *f = sctofunc(s);
 	    /* If the function associated with this shortcut is
 	     * cutting or copying text, remember this. */
 	    if (s->scfunc == do_cut_text_void
@@ -1701,34 +1711,35 @@ int do_input(bool allow_funcs)
 		)
 		preserve = TRUE;
 
-	    if (s->scfunc != 0) {
-		const subnfunc *f = sctofunc((sc *) s);
-		if (ISSET(VIEW_MODE) && f && !f->viewok)
-		    print_view_warning();
-		else {
+	    if (s->scfunc == NULL) {
+		statusbar("Internal error: shortcut without function!");
+		return ERR;
+	    }
+	    if (ISSET(VIEW_MODE) && f && !f->viewok)
+		print_view_warning();
+	    else {
 #ifndef NANO_TINY
-		    if (s->scfunc == do_toggle_void) {
-			do_toggle(s->toggle);
-			if (s->toggle != CUT_TO_END)
-			    preserve = TRUE;
-		    } else
+		if (s->scfunc == do_toggle_void) {
+		    do_toggle(s->toggle);
+		    if (s->toggle != CUT_TO_END)
+			preserve = TRUE;
+		} else
 #endif
-		    {
-			/* Execute the function of the shortcut. */
-			s->scfunc();
+		{
+		    /* Execute the function of the shortcut. */
+		    s->scfunc();
 #ifndef DISABLE_COLOR
-			if (f && !f->viewok && openfile->syntax != NULL
-				&& openfile->syntax->nmultis > 0)
-			    reset_multis(openfile->current, FALSE);
+		    if (f && !f->viewok)
+			reset_multis(openfile->current, FALSE);
 #endif
-			if (edit_refresh_needed) {
+		    if (edit_refresh_needed) {
 #ifdef DEBUG
-			    fprintf(stderr, "running edit_refresh() as edit_refresh_needed is true\n");
+			fprintf(stderr, "running edit_refresh() as edit_refresh_needed is true\n");
 #endif
-			    edit_refresh();
-			    edit_refresh_needed = FALSE;
-			}
-		    }
+			edit_refresh();
+			edit_refresh_needed = FALSE;
+		    } else if (s->scfunc == do_delete || s->scfunc == do_backspace)
+			update_line(openfile->current, openfile->current_x);
 		}
 	    }
 	}
@@ -1846,146 +1857,6 @@ int do_mouse(void)
 }
 #endif /* !DISABLE_MOUSE */
 
-#ifndef DISABLE_COLOR
-void alloc_multidata_if_needed(filestruct *fileptr)
-{
-    int i;
-
-    if (fileptr->multidata == NULL) {
-	fileptr->multidata = (short *)nmalloc(openfile->syntax->nmultis * sizeof(short));
-
-	for (i = 0; i < openfile->syntax->nmultis; i++)
-	    fileptr->multidata[i] = -1;
-    }
-}
-
-/* Precalculate the multi-line start and end regex info so we can
- * speed up rendering (with any hope at all...). */
-void precalc_multicolorinfo(void)
-{
-#ifdef DEBUG
-    fprintf(stderr, "Entering precalculation of multiline color info\n");
-#endif
-    if (openfile->colorstrings != NULL && !ISSET(NO_COLOR_SYNTAX)) {
-	const colortype *tmpcolor = openfile->colorstrings;
-	regmatch_t startmatch, endmatch;
-	filestruct *fileptr, *endptr;
-	time_t last_check = time(NULL), cur_check = 0;
-
-	/* Let us get keypresses to see if the user is trying to start
-	 * editing.  Later we may want to throw up a statusbar message
-	 * before starting this if it takes too long to do this routine.
-	 * For now silently abort if they hit a key. */
-	nodelay(edit, TRUE);
-
-	for (; tmpcolor != NULL; tmpcolor = tmpcolor->next) {
-
-	    /* If it's not a multi-line regex, amscray. */
-	    if (tmpcolor->end == NULL)
-		continue;
-#ifdef DEBUG
-	    fprintf(stderr, "Starting work on color id %d\n", tmpcolor->id);
-#endif
-
-	    for (fileptr = openfile->fileage; fileptr != NULL; fileptr = fileptr->next) {
-		int startx = 0;
-		int nostart = 0;
-#ifdef DEBUG
-		fprintf(stderr, "working on lineno %ld... ", (long)fileptr->lineno);
-#endif
-
-		alloc_multidata_if_needed(fileptr);
-
-		if ((cur_check = time(NULL)) - last_check > 1) {
-		    last_check = cur_check;
-		    if (wgetch(edit) != ERR)
-			goto precalc_cleanup;
-		}
-
-		while ((nostart = regexec(tmpcolor->start, &fileptr->data[startx], 1, &startmatch,
-					  (startx == 0) ? 0 : REG_NOTBOL)) == 0) {
-		    /* Look for an end, and start marking how many lines are
-		     * encompassed, which should speed up rendering later. */
-		    startx += startmatch.rm_eo;
-#ifdef DEBUG
-		    fprintf(stderr, "start found at pos %lu... ", (unsigned long)startx);
-#endif
-
-		    /* Look first on this line for an end. */
-		    if (regexec(tmpcolor->end, &fileptr->data[startx], 1, &endmatch,
-				(startx == 0) ? 0 : REG_NOTBOL) == 0) {
-			startx += endmatch.rm_eo;
-			fileptr->multidata[tmpcolor->id] = CSTARTENDHERE;
-#ifdef DEBUG
-			fprintf(stderr, "end found on this line\n");
-#endif
-			continue;
-		    }
-
-		    /* Nice, we didn't find the end regex on this line.  Let's start looking for it. */
-		    for (endptr = fileptr->next; endptr != NULL; endptr = endptr->next) {
-#ifdef DEBUG
-			fprintf(stderr, "\nadvancing to line %ld to find end... ", (long)endptr->lineno);
-#endif
-			/* Check for keyboard input, again. */
-			if ((cur_check = time(NULL)) - last_check > 1) {
-			    last_check = cur_check;
-			    if (wgetch(edit) != ERR)
-				goto precalc_cleanup;
-			}
-			if (regexec(tmpcolor->end, endptr->data, 1, &endmatch, 0) == 0)
-			    break;
-		    }
-
-		    if (endptr == NULL) {
-#ifdef DEBUG
-			fprintf(stderr, "no end found, breaking out\n");
-#endif
-			break;
-		    }
-
-#ifdef DEBUG
-		    fprintf(stderr, "end found\n");
-#endif
-		    /* We found it, we found it, la la la la la.  Mark all
-		     * the lines in between and the end properly. */
-		    fileptr->multidata[tmpcolor->id] = CENDAFTER;
-#ifdef DEBUG
-		    fprintf(stderr, "marking line %ld as CENDAFTER\n", (long)fileptr->lineno);
-#endif
-		    for (fileptr = fileptr->next; fileptr != endptr; fileptr = fileptr->next) {
-			alloc_multidata_if_needed(fileptr);
-			fileptr->multidata[tmpcolor->id] = CWHOLELINE;
-#ifdef DEBUG
-			fprintf(stderr, "marking intermediary line %ld as CWHOLELINE\n", (long)fileptr->lineno);
-#endif
-		    }
-		    alloc_multidata_if_needed(endptr);
-		    fileptr->multidata[tmpcolor->id] = CBEGINBEFORE;
-#ifdef DEBUG
-		    fprintf(stderr, "marking line %ld as CBEGINBEFORE\n", (long)fileptr->lineno);
-#endif
-		    /* Skip to the end point of the match. */
-		    startx = endmatch.rm_eo;
-#ifdef DEBUG
-		    fprintf(stderr, "jumping to line %ld pos %lu to continue\n", (long)fileptr->lineno, (unsigned long)startx);
-#endif
-		}
-		if (nostart && startx == 0) {
-#ifdef DEBUG
-		    fprintf(stderr, "no match\n");
-#endif
-		    fileptr->multidata[tmpcolor->id] = CNONE;
-		    continue;
-		}
-	    }
-	}
-    }
-precalc_cleanup:
-    nodelay(edit, FALSE);
-}
-#endif /* !DISABLE_COLOR */
-
 /* The user typed output_len multibyte characters.  Add them to the edit
  * buffer, filtering out all ASCII control characters if allow_cntrls is
  * TRUE. */
@@ -2074,14 +1945,6 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 #endif
     }
 
-#ifndef DISABLE_COLOR
-    /* If syntax highlighting is turned on and there are multiline regexes,
-     * the edit window will need to be refreshed. */
-    if (!ISSET(NO_COLOR_SYNTAX) && openfile->syntax &&
-		openfile->syntax->nmultis > 0)
-	edit_refresh_needed = TRUE;
-#endif
-
 #ifndef NANO_TINY
     /* Well, we might also need a full refresh if we've changed the
      * line length to be a new multiple of COLS. */
@@ -2097,6 +1960,7 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 #ifndef DISABLE_COLOR
     reset_multis(openfile->current, FALSE);
 #endif
+
     if (edit_refresh_needed == TRUE) {
 	edit_refresh();
 	edit_refresh_needed = FALSE;
@@ -2707,11 +2571,14 @@ int main(int argc, char **argv)
 #endif
 
 #if !defined(NANO_TINY) && defined(HAVE_KEY_DEFINED)
+    const char *keyvalue;
     /* Ask ncurses for the key codes for Control+Left and Control+Right. */
-    if ((int)tigetstr("kLFT5") > 0)
-	controlleft = key_defined(tigetstr("kLFT5"));
-    if ((int)tigetstr("kRIT5") > 0)
-	controlright = key_defined(tigetstr("kRIT5"));
+    keyvalue = tigetstr("kLFT5");
+    if (keyvalue != 0 && keyvalue != (char *)-1)
+	controlleft = key_defined(keyvalue);
+    keyvalue = tigetstr("kRIT5");
+    if (keyvalue != 0 && keyvalue != (char *)-1)
+	controlright = key_defined(keyvalue);
 #endif
 
 #ifdef DEBUG
@@ -2748,11 +2615,13 @@ int main(int argc, char **argv)
 	    if (i < argc - 1 && argv[i][0] == '+')
 		parse_line_column(&argv[i][1], &iline, &icol);
 	    else {
-		open_buffer(argv[i], FALSE);
+		/* If opening fails, don't try to position the cursor. */
+		if (!open_buffer(argv[i], FALSE))
+		    continue;
 
 		/* If a position was given on the command line, go there. */
 		if (iline > 0 || icol > 0) {
-		    do_gotolinecolumn(iline, icol, FALSE, FALSE, FALSE, FALSE);
+		    do_gotolinecolumn(iline, icol, FALSE, FALSE);
 		    iline = 0;
 		    icol = 0;
 		}
@@ -2761,8 +2630,8 @@ int main(int argc, char **argv)
 		    ssize_t savedposline, savedposcol;
 		    /* If edited before, restore the last cursor position. */
 		    if (check_poshistory(argv[i], &savedposline, &savedposcol))
-			do_gotolinecolumn(savedposline, savedposcol, FALSE,
-						FALSE, FALSE, FALSE);
+			do_gotolinecolumn(savedposline, savedposcol,
+						FALSE, FALSE);
 		}
 #endif
 	    }
@@ -2791,21 +2660,15 @@ int main(int argc, char **argv)
     fprintf(stderr, "Main: top and bottom win\n");
 #endif
 
-#ifndef DISABLE_COLOR
-    if (openfile->syntax)
-	if (openfile->syntax->nmultis > 0)
-	    precalc_multicolorinfo();
-#endif
-
     /* If a starting position was given on the command line, go there. */
     if (startline > 0 || startcol > 0)
-	do_gotolinecolumn(startline, startcol, FALSE, FALSE, FALSE, FALSE);
+	do_gotolinecolumn(startline, startcol, FALSE, FALSE);
 #ifndef DISABLE_HISTORIES
     else {
 	ssize_t savedposline, savedposcol;
 	/* If the file was edited before, restore the last cursor position. */
 	if (check_poshistory(argv[optind], &savedposline, &savedposcol))
-	    do_gotolinecolumn(savedposline, savedposcol, FALSE, FALSE, FALSE, FALSE);
+	    do_gotolinecolumn(savedposline, savedposcol, FALSE, FALSE);
     }
 #endif
 
