@@ -1,4 +1,4 @@
-/* $Id: prompt.c 5403 2015-11-11 19:56:35Z bens $ */
+/* $Id: prompt.c 5530 2016-01-04 09:52:43Z bens $ */
 /**************************************************************************
  *   prompt.c                                                             *
  *                                                                        *
@@ -210,7 +210,7 @@ int do_statusbar_input(bool *ran_func, bool *finished,
 		 * functions and setting finished to TRUE to indicate
 		 * that we're done after running or trying to run their
 		 * associated functions. */
-		f = sctofunc((sc *) s);
+		f = sctofunc(s);
 		if (s->scfunc != NULL) {
 		    *ran_func = TRUE;
 		    if (f && (!ISSET(VIEW_MODE) || f->viewok) &&
