@@ -1,4 +1,4 @@
-/* $Id: proto.h 5476 2015-12-04 11:01:48Z bens $ */
+/* $Id: proto.h 5532 2016-01-04 10:37:11Z bens $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -252,6 +252,9 @@ bool is_valid_mbstring(const char *s);
 void set_colorpairs(void);
 void color_init(void);
 void color_update(void);
+void reset_multis(filestruct *fileptr, bool force);
+void alloc_multidata_if_needed(filestruct *fileptr);
+void precalc_multicolorinfo(void);
 #endif
 
 /* All functions in cut.c. */
@@ -280,7 +283,7 @@ void do_uncut_text(void);
 /* All functions in files.c. */
 void make_new_buffer(void);
 void initialize_buffer_text(void);
-void open_buffer(const char *filename, bool undoable);
+bool open_buffer(const char *filename, bool undoable);
 #ifndef DISABLE_SPELLER
 void replace_buffer(const char *filename);
 #endif
@@ -291,8 +294,7 @@ void switch_to_prev_buffer_void(void);
 void switch_to_next_buffer_void(void);
 bool close_buffer(bool quiet);
 #endif
-filestruct *read_line(char *buf, filestruct *prevnode, bool
-	*first_line_ins, size_t buf_len);
+filestruct *read_line(char *buf, size_t buf_len, filestruct *prevnode);
 void read_file(FILE *f, int fd, const char *filename, bool undoable, bool checkwritable);
 int open_file(const char *filename, bool newfie, bool quiet, FILE **f);
 char *get_next_filename(const char *name, const char *suffix);
@@ -323,7 +325,7 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
 bool write_marked_file(const char *name, FILE *f_open, bool tmp,
 	append_type append);
 #endif
-bool do_writeout(bool exiting);
+int do_writeout(bool exiting);
 void do_writeout_void(void);
 #ifndef NANO_TINY
 void do_savefile(void);
@@ -368,10 +370,10 @@ void shortcut_init(void);
 void set_lint_or_format_shortcuts(void);
 void set_spell_shortcuts(void);
 #endif
-const subnfunc *sctofunc(sc *s);
+const subnfunc *sctofunc(const sc *s);
 const char *flagtostr(int flag);
-sc *strtosc(char *input);
-int strtomenu(char *input);
+sc *strtosc(const char *input);
+int strtomenu(const char *input);
 #ifdef DEBUG
 void thanks_for_all_the_fish(void);
 #endif
@@ -481,6 +483,7 @@ int more_space(void);
 int no_help(void);
 void no_current_file_name_warning(void);
 void do_exit(void);
+void close_and_go(void);
 void signal_init(void);
 RETSIGTYPE handle_hupterm(int signal);
 RETSIGTYPE do_suspend(int signal);
@@ -488,7 +491,7 @@ RETSIGTYPE do_continue(int signal);
 #ifndef NANO_TINY
 RETSIGTYPE handle_sigwinch(int signal);
 void regenerate_screen(void);
-void allow_pending_sigwinch(bool allow);
+void allow_sigwinch(bool allow);
 void do_toggle(int flag);
 #endif
 void do_toggle_void(void);
@@ -571,8 +574,6 @@ void parse_include(char *ptr);
 short color_to_short(const char *colorname, bool *bright);
 void parse_colors(char *ptr, bool icase);
 bool parse_color_names(char *combostr, short *fg, short *bg, bool *bright);
-void reset_multis(filestruct *fileptr, bool force);
-void alloc_multidata_if_needed(filestruct *fileptr);
 #endif
 void parse_rcfile(FILE *rcstream
 #ifndef DISABLE_COLOR
@@ -616,14 +617,10 @@ ssize_t do_replace_loop(
 	bool *canceled, const filestruct *real_current, size_t
 	*real_current_x, const char *needle);
 void do_replace(void);
-void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
-	bool interactive, bool save_pos, bool allow_update);
-void do_gotolinecolumn_void(void);
-#ifndef DISABLE_SPELLER
-void do_gotopos(ssize_t pos_line, size_t pos_x, ssize_t pos_y, size_t
-	pos_pww);
-#endif
 void goto_line_posx(ssize_t line, size_t pos_x);
+void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
+	bool interactive);
+void do_gotolinecolumn_void(void);
 #ifndef NANO_TINY
 bool find_bracket_match(bool reverse, const char *bracket_set);
 void do_find_bracket(void);
@@ -762,7 +759,7 @@ void dump_filestruct(const filestruct *inptr);
 void dump_filestruct_reverse(void);
 #endif
 
-/* All functions in winio.c. */
+/* Most functions in winio.c. */
 void get_key_buffer(WINDOW *win);
 size_t get_key_buffer_len(void);
 void unget_input(int *input, size_t input_len);
@@ -770,9 +767,8 @@ void unget_kbinput(int kbinput, bool metakey, bool funckey);
 int *get_input(WINDOW *win, size_t input_len);
 int get_kbinput(WINDOW *win);
 int parse_kbinput(WINDOW *win);
-int get_escape_seq_kbinput(const int *seq, size_t seq_len);
-int get_escape_seq_abcd(int kbinput);
-int parse_escape_seq_kbinput(WINDOW *win, int kbinput);
+int arrow_from_abcd(int kbinput);
+int parse_escape_sequence(WINDOW *win, int kbinput);
 int get_byte_kbinput(int kbinput);
 #ifdef ENABLE_UTF8
 long add_unicode_digit(int kbinput, long factor, long *uni);
@@ -837,6 +833,7 @@ void mac_format_void(void);
 void append_void(void);
 void prepend_void(void);
 void backup_file_void(void);
+void discard_buffer(void);
 void new_buffer_void(void);
 void backwards_void(void);
 void goto_dir_void(void);
