@@ -1,4 +1,4 @@
-/* $Id: search.c 5437 2015-11-24 13:24:01Z bens $ */
+/* $Id: search.c 5519 2015-12-31 19:20:40Z bens $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -236,8 +236,7 @@ int search_init(bool replacing, bool use_answer)
 		return -2;	/* Call the opposite search function. */
 	} else if (func == do_gotolinecolumn_void) {
 		do_gotolinecolumn(openfile->current->lineno,
-			openfile->placewewant + 1, TRUE, TRUE, FALSE,
-			TRUE);
+			openfile->placewewant + 1, TRUE, TRUE);
 				/* Put answer up on the statusbar and
 				 * fall through. */
 		return 3;
@@ -936,8 +935,8 @@ void do_replace(void)
 /* Go to the specified line and x position. */
 void goto_line_posx(ssize_t line, size_t pos_x)
 {
-    for (openfile->current = openfile->fileage; openfile->current != openfile->filebot &&
-					openfile->current->next != NULL && line > 1; line--)
+    for (openfile->current = openfile->fileage; line > 1 &&
+		openfile->current != openfile->filebot; line--)
 	openfile->current = openfile->current->next;
 
     openfile->current_x = pos_x;
@@ -947,11 +946,10 @@ void goto_line_posx(ssize_t line, size_t pos_x)
 }
 
 /* Go to the specified line and column, or ask for them if interactive
- * is TRUE.  Save the x-coordinate and y-coordinate if save_pos is TRUE.
- * Update the screen afterwards if allow_update is TRUE.  Note that both
- * the line and column numbers should be one-based. */
+ * is TRUE.  In the latter case also update the screen afterwards.
+ * Note that both the line and column number should be one-based. */
 void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
-	bool interactive, bool save_pos, bool allow_update)
+	bool interactive)
 {
     if (interactive) {
 	char *ans = mallocstrcpy(NULL, answer);
@@ -1013,13 +1011,11 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
     openfile->current_x = actual_x(openfile->current->data, column - 1);
     openfile->placewewant = column - 1;
 
-    /* Put the top line of the edit window in range of the current line.
-     * If save_pos is TRUE, don't change the cursor position when doing
-     * it. */
-    edit_update(save_pos ? NONE : CENTER);
+    /* Put the top line of the edit window in range of the current line. */
+    edit_update(CENTER);
 
-    /* If allow_update is TRUE, update the screen. */
-    if (allow_update) {
+    /* When in interactive mode, update the screen. */
+    if (interactive) {
 	edit_refresh();
 	display_main_list();
     }
@@ -1029,27 +1025,9 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 void do_gotolinecolumn_void(void)
 {
     do_gotolinecolumn(openfile->current->lineno,
-	openfile->placewewant + 1, FALSE, TRUE, FALSE, TRUE);
+	openfile->placewewant + 1, FALSE, TRUE);
 }
 
-#ifndef DISABLE_SPELLER
-/* Go to the line with the number specified in pos_line, the
- * x-coordinate specified in pos_x, the y-coordinate specified in pos_y,
- * and the place we want specified in pos_pww. */
-void do_gotopos(ssize_t pos_line, size_t pos_x, ssize_t pos_y, size_t
-	pos_pww)
-{
-    /* Since do_gotolinecolumn() resets the x-coordinate but not the
-     * y-coordinate, set the coordinates up this way. */
-    openfile->current_y = pos_y;
-    do_gotolinecolumn(pos_line, pos_x + 1, FALSE, FALSE, TRUE, TRUE);
-
-    /* Set the rest of the coordinates up. */
-    openfile->placewewant = pos_pww;
-    update_line(openfile->current, pos_x);
-}
-#endif
-
 #ifndef NANO_TINY
 /* Search for a match to one of the two characters in bracket_set.  If
  * reverse is TRUE, search backwards for the leftmost bracket.
